/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package conferencesolver;

import conferencesolver.problem.ConferenceObjective;
import conferencesolver.problem.ConferenceProblem;
import conferencesolver.problem.Objective;
import conferencesolver.solution.ConferenceSolution;
import java.io.File;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.Scanner;

/**
 *
 * @author Michael Brooks <mjbrooks@uw.edu>
 */
public class InputOutput {

    public void printSolution(ConferenceSolution solution) {
        System.out.println("organization:");

        ConferenceProblem problem = solution.getProblem();
        
        for (int s = 0; s < problem.getNumSessions(); s++) {
            for (int t = 0; t < problem.getTracksInSession(); t++) {
                for (int k = 0; k < problem.getPapersInSession(); k++) {
                    System.out.print(solution.getPaper(s, t, k) + " ");
                }
                if (t != problem.getTracksInSession() - 1) {
                    System.out.print("| ");
                }
            }
            System.out.println();
        }
    }

    public void printEvaluation(Objective objective, ConferenceSolution solution) {
        // Score the organization against the gold standard.
        double score = objective.scoreConference(solution);
        System.out.println("score:" + score);
    }
    
    /**
     * Read in the number of parallel tracks, papers in session, sessions in a
     * track, and the similarity matrix from the specified filename.
     *
     * @param filename is the name of the file containing the matrix.
     * @return the similarity matrix.
     * @throws FileNotFoundException
     */
    public ConferenceProblem readInInputFile(String filename)
            throws FileNotFoundException {
        Scanner in = new Scanner(new File(filename));
        ArrayList<String> lines = new ArrayList<String>();
        while (in.hasNextLine()) {
            String line = in.nextLine().trim();
            if (!line.equals("")) {
                lines.add(line);
            }
        }

        if (6 > lines.size()) {
            System.err.println("Not enough information given, check format of input file");
            System.exit(1);
        }
        
        double processingTimeInMinutes = Double.parseDouble(lines.get(0));
        int papersInSession = Integer.parseInt(lines.get(1));
        int tracksInSession = Integer.parseInt(lines.get(2));
        int numSessions = Integer.parseInt(lines.get(3));
        
        double tradeoffCoefficient = Double.parseDouble(lines.get(4));

        int n = lines.size() - 5;
        double[][] tempDistanceMatrix = new double[n][n];
        for (int i = 0; i < n; i++) {
            String tempLine = lines.get(i + 5);
            String[] elements = tempLine.split(" ");
            if (elements.length != n) {
                System.err.println(n + " " + elements.length);
                System.err.println("The similarity matrix does not have the correct format.");
                System.exit(1);
            }
            for (int j = 0; j < n; j++) {
                tempDistanceMatrix[i][j] = Double.parseDouble(elements[j].trim());
            }
        }
        
        Objective objective = new ConferenceObjective(tempDistanceMatrix, tradeoffCoefficient);
        ConferenceProblem problem = new ConferenceProblem(objective, numSessions, tracksInSession, papersInSession, processingTimeInMinutes);
        
        int numberOfPapers = tempDistanceMatrix.length;
        int slots = tracksInSession * papersInSession * numSessions;
        if (slots != numberOfPapers) {
            System.err.println("More papers than slots available! slots:" + slots + " num papers:" + numberOfPapers);
            System.exit(1);
        }

        return problem;
    }

    void printProblem(ConferenceProblem problem) {
        System.out.print("[S=" + problem.getNumSessions());
        System.out.print(" x T=" + problem.getTracksInSession());
        System.out.print(" x K=" + problem.getPapersInSession() + "]");
        System.out.println(" => " + problem.getTotalPapers());
    }
}
