package conferencesolver;

import conferencesolver.solution.improver.BasicLocalSearch;
import conferencesolver.problem.ConferenceProblem;
import conferencesolver.solution.ConferenceSolution;
import conferencesolver.solution.generator.*;
import conferencesolver.solution.improver.BasicMacroSearch;
import conferencesolver.solution.improver.FlipFlopSearch;
import conferencesolver.solution.improver.SolutionImprover;
import java.io.FileNotFoundException;

/**
 * SessionOrganizer reads in a similarity matrix of papers, and organizes them
 * into sessions and tracks.
 *
 * @author janara
 *
 */
public class SessionOrganizer {

    public static void main(String args[]) {
        long start = System.nanoTime();
        // Parse the input.
        if (args.length < 1) {
            System.err.println("SessionOrganizer <input_filename>");
            System.exit(1);
        }
        String inputfilename = args[0];
        String organizationfilename = "";
        if (args.length == 2) {
            organizationfilename = args[1];
        }


        // Initialize the conference organizer.
        InputOutput io = new InputOutput();
        ConferenceProblem problem = null;

        try {
            problem = io.readInInputFile(inputfilename);
        } catch (FileNotFoundException ex) {
            ex.printStackTrace();
            System.exit(1);
        }

        SolutionGenerator solver = null;
        if (organizationfilename.equals("")) {
            // Organize the papers into tracks based on similarity.
            SolutionImprover basicLocalTemplate = new BasicLocalSearch();
            SolutionImprover macroTemplate = new BasicMacroSearch();
            SolutionImprover flipTemplate = new FlipFlopSearch(basicLocalTemplate, macroTemplate);
            ScatterSolutionGenerator scatterGenerator = new ScatterSolutionGenerator();
            int allowedRestarts = 50000;
            solver = new RandomRestarts(allowedRestarts, flipTemplate, scatterGenerator);
            solver.setTimeLimit(problem.getProcessingTimeInMinutes() * 60);
        } else {
            // Read in an organization from a file
            solver = new FileSolutionGenerator(organizationfilename);
        }

        ConferenceSolution solution = solver.generateSolution(problem);

        System.out.println("BEST SOLUTION");
        io.printSolution(solution);
        io.printEvaluation(problem.getObjective(), solution);
        
        long stop = System.nanoTime();
        System.out.println("Time taken: " + ((stop - start) / 1000000000.0));
    }
}
