/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package conferencesolver;

import conferencesolver.problem.ConferenceProblem;
import conferencesolver.problem.RandomProblemGenerator;
import conferencesolver.solution.ConferenceSolution;
import conferencesolver.solution.generator.*;
import conferencesolver.solution.improver.*;
import java.io.FileNotFoundException;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Random;

/**
 *
 * @author Michael Brooks <mjbrooks@uw.edu>
 */
public class SolverEvaluator {

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {

        int maxSessions = 25;
        int maxTracks = 6;
        int maxPapersInSession = 4;
        int problemsToSolve = 20;
        double timeLimit = 5 * 60; // 5 minutes

        //Generate the problems
        int randomSeed = 567;
        Random random = new Random(randomSeed);
        ArrayList<ConferenceProblem> problems = new ArrayList<ConferenceProblem>();
        InputOutput io = new InputOutput();

        RandomProblemGenerator problemGenerator = new RandomProblemGenerator(random, maxSessions, maxTracks, maxPapersInSession);
//        for (int i = 0; i < problemsToSolve; i++) {
//            ConferenceProblem problem = problemGenerator.nextProblem();
//            problem.setName("Problem " + (i+1) + " (" + problem.getTotalPapers() + ")");
//            problems.add(problem);
//        }

        try {
            ConferenceProblem matrix1 = io.readInInputFile("sampleproblems/matrix1.txt");
            matrix1.setName("matrix1");
            problems.add(matrix1);

            ConferenceProblem matrix2 = io.readInInputFile("sampleproblems/matrix2.txt");
            matrix2.setName("matrix2");
            problems.add(matrix2);

            ConferenceProblem matrix3 = io.readInInputFile("sampleproblems/matrix3.txt");
            matrix3.setName("matrix3");
            problems.add(matrix3);

            ConferenceProblem matrix4 = io.readInInputFile("sampleproblems/matrix4.txt");
            matrix4.setName("matrix4");
            problems.add(matrix4);

            ConferenceProblem matrix5 = io.readInInputFile("sampleproblems/matrix5.txt");
            matrix5.setName("matrix5");
            problems.add(matrix5);

        } catch (FileNotFoundException e) {
            e.printStackTrace(System.err);
        }

        //Initialize the solvers to test
        ArrayList<SolutionGenerator> solutionGenerators = new ArrayList<SolutionGenerator>();

        //Local search templates
        SolutionImprover basicLocalTemplate = new BasicLocalSearch();
        int tabuListLength = 50;
        SolutionImprover tabuLocalTemplate = new TabuLocalSearch(tabuListLength);
        SolutionImprover macroTemplate = new BasicMacroSearch();
        SolutionImprover altTemplate = new AlternatingSearch(basicLocalTemplate, macroTemplate, true);
        SolutionImprover altRevTemplate = new AlternatingSearch(basicLocalTemplate, macroTemplate, false);
        SolutionImprover flipTemplate = new FlipFlopSearch(basicLocalTemplate, macroTemplate);
        SolutionImprover flipTabuTemplate = new FlipFlopSearch(tabuLocalTemplate, macroTemplate);

        //Starter generators
        RandomSolutionGenerator randomGenerator = new RandomSolutionGenerator(new Random());
        DumbSolutionGenerator dumbGenerator = new DumbSolutionGenerator();
        GreedySolutionGenerator greedyGenerator = new GreedySolutionGenerator();
        RGreedySolutionGenerator randomGreedyGenerator = new RGreedySolutionGenerator();
        ScatterSolutionGenerator scatterGenerator = new ScatterSolutionGenerator();

        int allowedRestarts = 30;

        //Random Restarts
        boolean randomRestarts = false;
        boolean oneStarts = true;

        //Local Search Variants
        boolean basicLocal = true;
        boolean tabuLocal = true;
        boolean macroLocal = false;
        boolean altLocal = true;
        boolean altRevLocal = true;
        boolean flipLocal = true;
        boolean flipTabuLocal = true;
        
        //Starters
        boolean randomStart = false;
        boolean dumbStart = false;
        boolean greedyStart = false;
        boolean rGreedyStart = false;
        boolean scatterStart = true;

        //Random restarters
        if (randomRestarts) {
            if (randomStart) {
                if (basicLocal) {
                    SolutionGenerator rrRandomBasic = new RandomRestarts(allowedRestarts, basicLocalTemplate, randomGenerator);
                    rrRandomBasic.setName("RR_Random_Basic");
                    solutionGenerators.add(rrRandomBasic);
                }
                if (tabuLocal) {
                    SolutionGenerator rrRandomTabu = new RandomRestarts(allowedRestarts, tabuLocalTemplate, randomGenerator);
                    rrRandomTabu.setName("RR_Random_Tabu");
                    solutionGenerators.add(rrRandomTabu);
                }
                if (macroLocal) {
                    SolutionGenerator rrRandomMacro = new RandomRestarts(allowedRestarts, macroTemplate, randomGenerator);
                    rrRandomMacro.setName("RR_Random_Macro");
                    solutionGenerators.add(rrRandomMacro);
                }
                if (altLocal) {
                    SolutionGenerator rrRandomAlt = new RandomRestarts(allowedRestarts, altTemplate, randomGenerator);
                    rrRandomAlt.setName("RR_Random_Alt");
                    solutionGenerators.add(rrRandomAlt);
                }
                if (altRevLocal) {
                    SolutionGenerator rrRandomAltRev = new RandomRestarts(allowedRestarts, altRevTemplate, randomGenerator);
                    rrRandomAltRev.setName("RR_Random_AltRev");
                    solutionGenerators.add(rrRandomAltRev);
                }
                if (flipLocal) {
                    SolutionGenerator rrRandomFlip = new RandomRestarts(allowedRestarts, flipTemplate, randomGenerator);
                    rrRandomFlip.setName("RR_Random_Flip");
                    solutionGenerators.add(rrRandomFlip);
                }
                if (flipTabuLocal) {
                    SolutionGenerator rrRandomFlipTabu = new RandomRestarts(allowedRestarts, flipTabuTemplate, randomGenerator);
                    rrRandomFlipTabu.setName("RR_Random_FlipTabu");
                    solutionGenerators.add(rrRandomFlipTabu);
                }
            }
            if (scatterStart) {
                if (basicLocal) {
                    SolutionGenerator rrScatterBasic = new RandomRestarts(allowedRestarts, basicLocalTemplate, scatterGenerator);
                    rrScatterBasic.setName("RR_Scatter_Basic");
                    solutionGenerators.add(rrScatterBasic);
                }
                if (tabuLocal) {
                    SolutionGenerator rrScatterTabu = new RandomRestarts(allowedRestarts, tabuLocalTemplate, scatterGenerator);
                    rrScatterTabu.setName("RR_Scatter_Tabu");
                    solutionGenerators.add(rrScatterTabu);
                }
                if (macroLocal) {
                    SolutionGenerator rrScatterMacro = new RandomRestarts(allowedRestarts, macroTemplate, scatterGenerator);
                    rrScatterMacro.setName("RR_Scatter_Macro");
                    solutionGenerators.add(rrScatterMacro);
                }
                if (altLocal) {
                    SolutionGenerator rrScatterAlt = new RandomRestarts(allowedRestarts, altTemplate, scatterGenerator);
                    rrScatterAlt.setName("RR_Scatter_Alt");
                    solutionGenerators.add(rrScatterAlt);
                }
                if (altRevLocal) {
                    SolutionGenerator rrScatterAltRev = new RandomRestarts(allowedRestarts, altRevTemplate, scatterGenerator);
                    rrScatterAltRev.setName("RR_Scatter_AltRev");
                    solutionGenerators.add(rrScatterAltRev);
                }
                if (flipLocal) {
                    SolutionGenerator rrScatterFlip = new RandomRestarts(allowedRestarts, flipTemplate, scatterGenerator);
                    rrScatterFlip.setName("RR_Scatter_Flip");
                    solutionGenerators.add(rrScatterFlip);
                }
                if (flipTabuLocal) {
                    SolutionGenerator rrScatterFlipTabu = new RandomRestarts(allowedRestarts, flipTabuTemplate, scatterGenerator);
                    rrScatterFlipTabu.setName("RR_Scatter_FlipTabu");
                    solutionGenerators.add(rrScatterFlipTabu);
                }
            }
            if (rGreedyStart) {
                if (basicLocal) {
                    SolutionGenerator rrRGreedyBasic = new RandomRestarts(allowedRestarts, basicLocalTemplate, randomGreedyGenerator);
                    rrRGreedyBasic.setName("RR_RGreedy_Basic");
                    solutionGenerators.add(rrRGreedyBasic);
                }
                if (tabuLocal) {
                    SolutionGenerator rrRGreedyTabu = new RandomRestarts(allowedRestarts, tabuLocalTemplate, randomGreedyGenerator);
                    rrRGreedyTabu.setName("RR_RGreedy_Tabu");
                    solutionGenerators.add(rrRGreedyTabu);
                }
                if (macroLocal) {
                    SolutionGenerator rrRGreedyMacro = new RandomRestarts(allowedRestarts, macroTemplate, randomGreedyGenerator);
                    rrRGreedyMacro.setName("RR_RGreedy_Macro");
                    solutionGenerators.add(rrRGreedyMacro);
                }
                if (altLocal) {
                    SolutionGenerator rrRGreedyAlt = new RandomRestarts(allowedRestarts, altTemplate, randomGreedyGenerator);
                    rrRGreedyAlt.setName("RR_RGreedy_Alt");
                    solutionGenerators.add(rrRGreedyAlt);
                }
                if (altRevLocal) {
                    SolutionGenerator rrRGreedyAltRev = new RandomRestarts(allowedRestarts, altRevTemplate, randomGreedyGenerator);
                    rrRGreedyAltRev.setName("RR_RGreedy_AltRev");
                    solutionGenerators.add(rrRGreedyAltRev);
                }
                if (flipLocal) {
                    SolutionGenerator rrRGreedyFlip = new RandomRestarts(allowedRestarts, flipTemplate, randomGreedyGenerator);
                    rrRGreedyFlip.setName("RR_RGreedy_Flip");
                    solutionGenerators.add(rrRGreedyFlip);
                }
                if (flipTabuLocal) {
                    SolutionGenerator rrRGreedyFlipTabu = new RandomRestarts(allowedRestarts, flipTabuTemplate, randomGreedyGenerator);
                    rrRGreedyFlipTabu.setName("RR_RGreedy_FlipTabu");
                    solutionGenerators.add(rrRGreedyFlipTabu);
                }
            }
        }
        if (oneStarts) {
            if (dumbStart) {
                if (basicLocal) {
                    SolutionGenerator oneDumbBasic = new OneStart(basicLocalTemplate, dumbGenerator);
                    oneDumbBasic.setName("One_Dumb_Basic");
                    solutionGenerators.add(oneDumbBasic);
                }
                if (tabuLocal) {
                    SolutionGenerator oneDumbTabu = new OneStart(tabuLocalTemplate, dumbGenerator);
                    oneDumbTabu.setName("One_Dumb_Tabu");
                    solutionGenerators.add(oneDumbTabu);
                }
                if (macroLocal) {
                    SolutionGenerator oneDumbMacro = new OneStart(macroTemplate, dumbGenerator);
                    oneDumbMacro.setName("One_Dumb_Macro");
                    solutionGenerators.add(oneDumbMacro);
                }
                if (altLocal) {
                    SolutionGenerator oneDumbAlt = new OneStart(altTemplate, dumbGenerator);
                    oneDumbAlt.setName("One_Dumb_Alt");
                    solutionGenerators.add(oneDumbAlt);
                }
                if (altRevLocal) {
                    SolutionGenerator oneDumbAltRev = new OneStart(altRevTemplate, dumbGenerator);
                    oneDumbAltRev.setName("One_Dumb_AltRev");
                    solutionGenerators.add(oneDumbAltRev);
                }
                if (flipLocal) {
                    SolutionGenerator oneDumbFlip = new OneStart(flipTemplate, dumbGenerator);
                    oneDumbFlip.setName("One_Dumb_Flip");
                    solutionGenerators.add(oneDumbFlip);
                }
                if (flipTabuLocal) {
                    SolutionGenerator oneDumbFlipTabu = new OneStart(flipTabuTemplate, dumbGenerator);
                    oneDumbFlipTabu.setName("One_Dumb_FlipTabu");
                    solutionGenerators.add(oneDumbFlipTabu);
                }
            }
            if (randomStart) {
                if (basicLocal) {
                    SolutionGenerator oneRandomBasic = new OneStart(basicLocalTemplate, randomGenerator);
                    oneRandomBasic.setName("One_Random_Basic");
                    solutionGenerators.add(oneRandomBasic);
                }
                if (tabuLocal) {
                    SolutionGenerator oneRandomTabu = new OneStart(tabuLocalTemplate, randomGenerator);
                    oneRandomTabu.setName("One_Random_Tabu");
                    solutionGenerators.add(oneRandomTabu);
                }
                if (macroLocal) {
                    SolutionGenerator oneRandomMacro = new OneStart(macroTemplate, randomGenerator);
                    oneRandomMacro.setName("One_Random_Macro");
                    solutionGenerators.add(oneRandomMacro);
                }
                if (altLocal) {
                    SolutionGenerator oneRandomAlt = new OneStart(altTemplate, randomGenerator);
                    oneRandomAlt.setName("One_Random_Alt");
                    solutionGenerators.add(oneRandomAlt);
                }
                if (altRevLocal) {
                    SolutionGenerator oneRandomAltRev = new OneStart(altRevTemplate, randomGenerator);
                    oneRandomAltRev.setName("One_Random_AltRev");
                    solutionGenerators.add(oneRandomAltRev);
                }
                if (flipLocal) {
                    SolutionGenerator oneRandomFlip = new OneStart(flipTemplate, randomGenerator);
                    oneRandomFlip.setName("One_Random_Flip");
                    solutionGenerators.add(oneRandomFlip);
                }
                if (flipTabuLocal) {
                    SolutionGenerator oneRandomFlipTabu = new OneStart(flipTabuTemplate, randomGenerator);
                    oneRandomFlipTabu.setName("One_Random_FlipTabu");
                    solutionGenerators.add(oneRandomFlipTabu);
                }
            }
            if (greedyStart) {
                if (basicLocal) {
                    SolutionGenerator oneGreedyBasic = new OneStart(basicLocalTemplate, greedyGenerator);
                    oneGreedyBasic.setName("One_Greedy_Basic");
                    solutionGenerators.add(oneGreedyBasic);
                }
                if (tabuLocal) {
                    SolutionGenerator oneGreedyTabu = new OneStart(tabuLocalTemplate, greedyGenerator);
                    oneGreedyTabu.setName("One_Greedy_Tabu");
                    solutionGenerators.add(oneGreedyTabu);
                }
                if (macroLocal) {
                    SolutionGenerator oneGreedyMacro = new OneStart(macroTemplate, greedyGenerator);
                    oneGreedyMacro.setName("One_Greedy_Macro");
                    solutionGenerators.add(oneGreedyMacro);
                }
                if (altLocal) {
                    SolutionGenerator oneGreedyAlt = new OneStart(altTemplate, greedyGenerator);
                    oneGreedyAlt.setName("One_Greedy_Alt");
                    solutionGenerators.add(oneGreedyAlt);
                }
                if (altRevLocal) {
                    SolutionGenerator oneGreedyAltRev = new OneStart(altRevTemplate, greedyGenerator);
                    oneGreedyAltRev.setName("One_Greedy_AltRev");
                    solutionGenerators.add(oneGreedyAltRev);
                }
                if (flipLocal) {
                    SolutionGenerator oneGreedyFlip = new OneStart(flipTemplate, greedyGenerator);
                    oneGreedyFlip.setName("One_Greedy_Flip");
                    solutionGenerators.add(oneGreedyFlip);
                }
                if (flipTabuLocal) {
                    SolutionGenerator oneGreedyFlipTabu = new OneStart(flipTabuTemplate, greedyGenerator);
                    oneGreedyFlipTabu.setName("One_Greedy_FlipTabu");
                    solutionGenerators.add(oneGreedyFlipTabu);
                }
            }
            if (rGreedyStart) {
                if (basicLocal) {
                    SolutionGenerator oneRGreedyBasic = new OneStart(basicLocalTemplate, randomGreedyGenerator);
                    oneRGreedyBasic.setName("One_RGreedy_Basic");
                    solutionGenerators.add(oneRGreedyBasic);
                }
                if (tabuLocal) {
                    SolutionGenerator oneRGreedyTabu = new OneStart(tabuLocalTemplate, randomGreedyGenerator);
                    oneRGreedyTabu.setName("One_RGreedy_Tabu");
                    solutionGenerators.add(oneRGreedyTabu);
                }
                if (macroLocal) {
                    SolutionGenerator oneRGreedyMacro = new OneStart(macroTemplate, randomGreedyGenerator);
                    oneRGreedyMacro.setName("One_RGreedy_Macro");
                    solutionGenerators.add(oneRGreedyMacro);
                }
                if (altLocal) {
                    SolutionGenerator oneRGreedyAlt = new OneStart(altTemplate, randomGreedyGenerator);
                    oneRGreedyAlt.setName("One_RGreedy_Alt");
                    solutionGenerators.add(oneRGreedyAlt);
                }
                if (altRevLocal) {
                    SolutionGenerator oneRGreedyAltRev = new OneStart(altRevTemplate, randomGreedyGenerator);
                    oneRGreedyAltRev.setName("One_RGreedy_AltRev");
                    solutionGenerators.add(oneRGreedyAltRev);
                }
                if (flipLocal) {
                    SolutionGenerator oneRGreedyFlip = new OneStart(flipTemplate, randomGreedyGenerator);
                    oneRGreedyFlip.setName("One_RGreedy_Flip");
                    solutionGenerators.add(oneRGreedyFlip);
                }
                if (flipTabuLocal) {
                    SolutionGenerator oneRGreedyFlipTabu = new OneStart(flipTabuTemplate, randomGreedyGenerator);
                    oneRGreedyFlipTabu.setName("One_RGreedy_FlipTabu");
                    solutionGenerators.add(oneRGreedyFlipTabu);
                }
            }
            if (scatterStart) {
                if (basicLocal) {
                    SolutionGenerator oneScatterBasic = new OneStart(basicLocalTemplate, scatterGenerator);
                    oneScatterBasic.setName("One_Scatter_Basic");
                    solutionGenerators.add(oneScatterBasic);
                }
                if (tabuLocal) {
                    SolutionGenerator oneScatterTabu = new OneStart(tabuLocalTemplate, scatterGenerator);
                    oneScatterTabu.setName("One_Scatter_Tabu");
                    solutionGenerators.add(oneScatterTabu);
                }
                if (macroLocal) {
                    SolutionGenerator oneScatterMacro = new OneStart(macroTemplate, scatterGenerator);
                    oneScatterMacro.setName("One_Scatter_Macro");
                    solutionGenerators.add(oneScatterMacro);
                }
                if (altLocal) {
                    SolutionGenerator oneScatterAlt = new OneStart(altTemplate, scatterGenerator);
                    oneScatterAlt.setName("One_Scatter_Alt");
                    solutionGenerators.add(oneScatterAlt);
                }
                if (altRevLocal) {
                    SolutionGenerator oneScatterAltRev = new OneStart(altRevTemplate, scatterGenerator);
                    oneScatterAltRev.setName("One_Scatter_AltRev");
                    solutionGenerators.add(oneScatterAltRev);
                }
                if (flipLocal) {
                    SolutionGenerator oneScatterFlip = new OneStart(flipTemplate, scatterGenerator);
                    oneScatterFlip.setName("One_Scatter_Flip");
                    solutionGenerators.add(oneScatterFlip);
                }
                if (flipTabuLocal) {
                    SolutionGenerator oneScatterFlipTabu = new OneStart(flipTabuTemplate, scatterGenerator);
                    oneScatterFlipTabu.setName("One_Scatter_FlipTabu");
                    solutionGenerators.add(oneScatterFlipTabu);
                }
            }
        }


        PrintStream csv = null;
        String outputFile = "results." + System.currentTimeMillis() + ".csv";
        System.out.println("Testing " + solutionGenerators.size() + " solvers on " + problems.size() + " problems.");
        System.out.println("Writing to " + outputFile);

        try {
            csv = new PrintStream(outputFile);
        } catch (FileNotFoundException e) {
            e.printStackTrace(System.err);
            System.exit(1);
        }

        Evaluation evaluation = new Evaluation();
        evaluation.printCSVHeader(csv);

        int repetitions = 30;

        //On each problem
        for (int i = 0; i < problems.size(); i++) {
            ConferenceProblem problem = problems.get(i);
            System.out.println("Testing problem " + problem.getName() + ", with " + problem.getTotalPapers() + " papers... ");

            //Test each solver
            for (int s = 0; s < solutionGenerators.size(); s++) {

                for (int rep = 0; rep < repetitions; rep++) {
                    SolutionGenerator improver = solutionGenerators.get(s);
                    System.out.print("  Testing solver " + improver.getName() + " rep " + (rep + 1) + "... ");

                    improver.doEvaluation(evaluation);
                    improver.setTimeLimit(timeLimit);
                    improver.setRepetition(rep);
                    
                    ConferenceSolution solution = improver.generateSolution(problem);

                    System.out.println(" reached " + improver.getCurrentScore() + " in " + improver.getTimeTaken() + " seconds");

                    evaluation.printCSV(csv);
                    evaluation.clear();
                    csv.flush();
                }
            }
        }

        csv.close();

    }
//    static class RandomRestartsMetrics {
//
//        ConferenceProblem problem;
//        private final AbstractSolutionImprover agent;
//        
//        private static void printHeaderCSV() {
//            System.out.print("ProblemSize,");
//            System.out.print("Sessions,");
//            System.out.print("Tracks,");
//            System.out.print("PapersInSession,");
//
//            System.out.print("AllowedShoulderTravel,");
//            System.out.print("TabuListSize,");
//            System.out.print("KickProbability,");
//
//            System.out.print("ScoreImprovementPercent,");
//            System.out.print("StepsTaken,");
//            System.out.print("TimeTaken(ms),");
//            System.out.print("TabuCollisions,");
//            System.out.print("ShoulderSteps,");
//            System.out.print("Kicks,");
//            
//            System.out.println();
//        }
//
//        
//        private void printCSV() {
//            System.out.print(problem.getTotalPapers() + ",");
//            System.out.print(problem.getNumSessions() + ",");
//            System.out.print(problem.getTracksInSession() + ",");
//            System.out.print(problem.getPapersInSession() + ",");
//
//            System.out.print(agent.getAllowedShoulderTravel() + ",");
//            System.out.print(agent.getTabuListSize() + ",");
//            System.out.print(agent.getProbabilityOfKick() + ",");
//
//            double scoreImprovement = agent.getScoreImprovement();
//            scoreImprovement /= agent.getStartingScore();
//            System.out.print(scoreImprovement + ",");
//            System.out.print(agent.getStepsTaken() + ",");
//            System.out.print(agent.getTimeTaken() + ",");
//            System.out.print(agent.getTabuCollisions() + ",");
//            System.out.print(agent.getShoulderTravelled() + ",");
//            System.out.print(agent.getNumKicks() + ",");
//            
//            System.out.println();
//        }
//        
//        private RandomRestartsMetrics(ConferenceProblem problem, AbstractSolutionImprover agent) {
//            this.problem = problem;
//            this.agent = agent;
//        }
//
//
//        
//        
//    }
}
