/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package conferencesolver.problem;

import java.util.Random;

/**
 *
 * @author Michael Brooks <mjbrooks@uw.edu>
 */
public class RandomProblemGenerator {

    private final Random random;
    private final int maxSessions;
    private final int maxTracks;
    private final int maxPapersInsession;

    public RandomProblemGenerator(Random random, int maxSessions, int maxTracks, int maxPapersInSession) {
        this.random = random;
        this.maxSessions = maxSessions;
        this.maxTracks = maxTracks;
        this.maxPapersInsession = maxPapersInSession;
    }

    public RandomProblemGenerator() {
        this(new Random(), 5, 6, 6);
    }

    public RandomProblemGenerator(Random random) {
        this(random, 5, 6, 6);
    }

    public ConferenceProblem nextProblem() {

        int papersInSession = 2 + random.nextInt(maxPapersInsession - 1);
        int tracksInSession = 2 + random.nextInt(maxTracks - 1);
        int numSessions = 1 + random.nextInt(maxSessions);

        int totalPapers = papersInSession * tracksInSession * numSessions;

        double[][] distanceMatrix = new double[totalPapers][totalPapers];

        for (int p1 = 0; p1 < totalPapers; p1++) {
            distanceMatrix[p1][p1] = 0;

            for (int p2 = p1 + 1; p2 < totalPapers; p2++) {
                double distance = random.nextDouble();
                distanceMatrix[p1][p2] = distance;
                distanceMatrix[p2][p1] = distance;
            }
        }

        double tradeoffCoefficient = 1;
        double processingTime = 5;//minutes
        ConferenceObjective objective = new ConferenceObjective(distanceMatrix, tradeoffCoefficient);
        ConferenceProblem problem = new ConferenceProblem(objective, numSessions, tracksInSession, papersInSession, processingTime);

        return problem;
    }
}
