package conferencesolver.solution;

/**
 * Session maintains an array of tracks that all take place simultaneously.
 *
 * @author janara
 *
 */
public class Session {

    // The array of tracks.
    Track[] tracks;

    public Session(int numberOfTracks) {
        tracks = new Track[numberOfTracks];
    }

    /**
     * Set the paper at the slot index to the specified paper id number.
     *
     * @param sessionIndex is the session to modify.
     * @param paperIndex is the index of the paper.
     * @param paperId is the id number of the paper.
     * @throws IndexOutOfBoundsException
     */
    public void setPaper(int trackIndex,
            int paperIndex,
            int paperId)
            throws IndexOutOfBoundsException {
        if (tracks.length > trackIndex) {
            Track curSession = tracks[trackIndex];
            curSession.setPaper(paperIndex, paperId);
        } else {
            throw new IndexOutOfBoundsException();
        }
    }
    
    public void removePaper(int trackIndex, int paperIndex) {
        if (tracks.length > trackIndex) {
            Track curSession = tracks[trackIndex];
            curSession.removePaper(paperIndex);
        } else {
            throw new IndexOutOfBoundsException();
        }
    }

    /**
     * Get the number of tracks in the session.
     *
     * @return the number of tracks in the session.
     */
    public int getNumberOfTracks() {
        return tracks.length;
    }

    /**
     * Get a specified track.
     *
     * @param index is the index of the track in question.
     * @return the track at the index.
     * @throws IndexOutOfBoundsException
     */
    public Track getTrack(int index) throws IndexOutOfBoundsException {
        if (index < tracks.length) {
            return tracks[index];
        } else {
            throw new IndexOutOfBoundsException();
        }
    }

    /**
     * Set the session at the specified index to the new session.
     *
     * @param index is the index of the session.
     * @param session is the new session
     * @throws IndexOutOfBoundsException
     */
    public void setTrack(int index,
            Track session)
            throws IndexOutOfBoundsException {
        if (index < tracks.length) {
            tracks[index] = session;
        } else {
            throw new IndexOutOfBoundsException();
        }
    }
}
