package conferencesolver.solution;


/**
 * Session holds an array of papers.
 * 
 * @author janara
 *
 */
public class Track {
    
    // The array of papers.
    int[] papers;
    
    public Track(int papersInSession) {
        initPapers(papersInSession);
    }
    
    /**
     * Initialize the array of papers.
     * 
     * @param papersInSession the number of papers in a session*track.
     */
    private void initPapers(int papersInSession) {
        papers = new int[papersInSession];
        for (int i = 0; i < papersInSession; i++) {
            papers[i] = -1;
        }
    }
    
    /**
     * Get the number of papers in a session.
     * 
     * @return the number of papers.
     */
    public int getNumberOfPapers() {
        return papers.length;
    }
    
    /**
     * Get the id of the paper at the specified index
     * 
     * @param index the index of the paper
     * @return the id of the paper
     * @throws IndexOutOfBoundsException
     */
    public int getPaper(int index) throws IndexOutOfBoundsException {
        if (index < papers.length) {
            return papers[index];
        } else {
            throw new IndexOutOfBoundsException();
        }
    }
    
    /**
     * Set the paper id at the specified index.
     * 
     * @param index is the index in the array
     * @param paperId is the id of the paper
     * @throws IndexOutOfBoundsException
     */
    public void setPaper(int index, int paperId) 
            throws IndexOutOfBoundsException {
        if (index < papers.length) {
            papers[index] = paperId;
        } else {
            throw new IndexOutOfBoundsException();
        }
    }
    
    public void removePaper(int index) {
        this.setPaper(index, -1);
    }
    
    public String toString() {
        String string = "";
        for (int i = 0; i < papers.length; i++) {
            string += papers[i]+" ";
        }
        return string;
    }
}
