/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package conferencesolver.solution.generator;

import conferencesolver.Evaluation;
import conferencesolver.problem.ConferenceProblem;
import conferencesolver.problem.Objective;
import conferencesolver.solution.ConferenceSolution;

/**
 *
 * @author Michael Brooks <mjbrooks@uw.edu>
 */
public abstract class AbstractSolutionGenerator implements SolutionGenerator {

    private double currentBestScore;
    private double timeTaken;
    private long startTime;
    private Evaluation evaluation;
    protected Objective objective;
    private ConferenceSolution currentBestSolution;
    private double timeLimit;
    protected ConferenceProblem problem;
    private String name;
    private int repetition;

    public abstract void internalGenerateSolution(ConferenceProblem problem);

    @Override
    public ConferenceSolution generateSolution(ConferenceProblem problem) {
        this.start();

        this.problem = problem;
        this.objective = problem.getObjective();

        internalGenerateSolution(problem);

        update(this.currentBestSolution);

        return this.currentBestSolution;
    }

    protected void start() {
        this.currentBestSolution = null;
        this.currentBestScore = -Double.MAX_VALUE;
        this.startTime = System.nanoTime();
        this.timeTaken = 0;
    }

    protected boolean update(ConferenceSolution solution) {
        double solutionQuality = objective.scoreConference(solution);
        return update(solution, solutionQuality);
    }

    protected boolean update(ConferenceSolution solution, double solutionQuality) {

        long stopTime = System.nanoTime();
        timeTaken = (stopTime - startTime) / 1000000000.0;


        if (this.currentBestScore < solutionQuality) {
            this.currentBestScore = solutionQuality;
            this.currentBestSolution = solution.deepCopy();

            if (evaluation != null) {
                evaluation.reportProgress(problem.getName(), this.getName(), repetition, timeTaken, currentBestScore);
            }
        }

        if (timeTaken > timeLimit) {
            return false;
        } else {
            return true;
        }
    }

    @Override
    public void doEvaluation(Evaluation evaluation) {
        this.evaluation = evaluation;
    }

    @Override
    public void setTimeLimit(double timeLimit) {
        this.timeLimit = timeLimit;
    }

    protected double getTimeLimit() {
        return this.timeLimit;
    }

    @Override
    public double getCurrentScore() {
        return currentBestScore;
    }

    @Override
    public double getTimeTaken() {
        return timeTaken;
    }

    @Override
    public String getName() {
        return this.name;
    }

    @Override
    public void setName(String name) {
        this.name = name;
    }

    public void setRepetition(int rep) {
        this.repetition = rep;
    }
}
