/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package conferencesolver.solution.generator;

import conferencesolver.problem.ConferenceProblem;
import conferencesolver.solution.ConferenceSolution;

/**
 *
 * @author Michael Brooks <mjbrooks@uw.edu>
 */
public class GlobalGreedySolutionGenerator extends AbstractSolutionGenerator {
    private ConferenceSolution solution;

    @Override
    public void internalGenerateSolution(ConferenceProblem problem) {
        
//        int clusterSize = problem.getPapersInSession();
//        double[] paperExtremism = getPaperExtremism(problem);
        
        solution = new ConferenceSolution(problem);
        
//        int[] paperAssignments = new int[problem.getTotalPapers()];
        
//        int paperCounter = 0;
        for (int s = 0; s < problem.getNumSessions(); s++) {
            for (int t = 0; t < problem.getTracksInSession(); t++) {
                
                for (int k = 0; k < problem.getPapersInSession(); k++) {
                    int slot = solution.getGlobalSlot(s, t, k);
                    int bestPaper = getBestPaperFor(slot);
                    solution.setPaper(slot, bestPaper);
                }
                
            }
        }
        
        update(solution);
    }
    
    private int getBestPaperFor(int slot) {
        double bestScore = -Double.MAX_VALUE;
        int bestPaper = -1;
        for (int p = 0; p < problem.getTotalPapers(); p++) {
            //Make sure p isn't already assigned
            if (solution.getGlobalSlot(p) != -1) {
                continue;
            }
            
            solution.setPaper(slot, p);
            double score = objective.getValueFromSlot(solution, slot);
            solution.removePaper(slot, p);
            
            if (score > bestScore) {
                bestScore = score;
                bestPaper = p;
            }
        }
        
        return bestPaper;
    }
//    private double[] getPaperExtremism(ConferenceProblem problem) {
//        double[] extremism = new double[problem.getTotalPapers()];
//        
//        Objective objective = problem.getObjective();
//        
//        for (int p1 = 0; p1 < problem.getTotalPapers(); p1++) {
//            for (int p2 = p1 + 1; p2 < problem.getTotalPapers(); p2++) {
//                
//                double distance = objective.getDistance(p1, p2);
//                
//                //tranform it into an extremism metric
//                distance -= 0.5;
//                distance = distance * distance;
//                
//                extremism[p1] += distance;
//                extremism[p2] += distance;
//            }
//            
//            //We're done with p1 now
//            extremism[p1] /= problem.getTotalPapers();
//        }
//        
//        return extremism;
//    }

    private int getPaperMostDistantFor(int session, int track) {
        
        int startSlot = solution.getGlobalSlot(session, 0, 0);
        int stopSlot = solution.getGlobalSlot(session, track - 1, problem.getPapersInSession() - 1);
        
        int bestPaper = -1;
        double bestScore = -1;
        
        for (int p1 = 0; p1 < problem.getTotalPapers(); p1++) {
            int currentSlot = solution.getGlobalSlot(p1);
            if (currentSlot != -1) {
                continue; //already assigned
            }
            
            double score = 0;
            //Test against every paper already assigned to this session
            for (int slot = startSlot; slot < stopSlot; slot++) {
                int placedPaper = solution.getPaper(slot);
                score += objective.getDistance(p1, placedPaper);
            }
            
            if (score > bestScore) {
                bestPaper = p1;
                bestScore = score;
            }
        }
        
        return bestPaper;
    }

    private int getPaperClosestTo(int session, int track) {
        int bestPaper = -1;
        double bestScore = -1;
        
        for (int p1 = 0; p1 < problem.getTotalPapers(); p1++) {
            int currentSlot = solution.getGlobalSlot(p1);
            if (currentSlot != -1) {
                continue; //already assigned
            }
            
            double score = 0;
            
            for (int k = 0; k < problem.getPapersInSession(); k++) {
                int paperInSession = solution.getPaper(session, track, k);
                if (paperInSession == -1) {
                    continue;
                }
                
                score += 1 - objective.getDistance(p1, paperInSession);
            }
            
            if (score > bestScore) {
                bestPaper = p1;
                bestScore = score;
            }
        }
        
        return bestPaper;
    }
}
