/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package conferencesolver.solution.generator;

import conferencesolver.problem.ConferenceProblem;
import conferencesolver.solution.ConferenceSolution;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Random;

/**
 *
 * @author Michael Brooks <mjbrooks@uw.edu>
 */
public class RGreedySolutionGenerator extends AbstractSolutionGenerator {
    private ConferenceSolution solution;

    @Override
    public void internalGenerateSolution(ConferenceProblem problem) {
        
//        int clusterSize = problem.getPapersInSession();
//        double[] paperExtremism = getPaperExtremism(problem);
        
        solution = new ConferenceSolution(problem);
        
//        int[] paperAssignments = new int[problem.getTotalPapers()];
        
//        int paperCounter = 0;
        
        ArrayList<Integer> sessionList = new ArrayList<Integer>(problem.getNumSessions());
        for (int s = 0; s < problem.getNumSessions(); s++) {
            sessionList.add(s);
        }
        
        Random random = new Random();
        
        for (int t = 0; t < problem.getTracksInSession(); t++) {
            //Go through the sessions randomly
            Collections.shuffle(sessionList, random);
            
            for (int i = 0; i < sessionList.size(); i++) {
                int s = sessionList.get(i);
                
                //Choose the seed for this session*track
                int seedPaper = getPaperMostDistantFor(s, t);
                solution.setPaper(s, t, 0, seedPaper);
                
                //Choose the rest of the papers
                for (int k = 1; k < problem.getPapersInSession(); k++) {
                    int bestMatch = getPaperClosestTo(s, t);
                    solution.setPaper(s, t, k, bestMatch);
                }
            }
        }
        
        update(solution);
    }
    

//    private double[] getPaperExtremism(ConferenceProblem problem) {
//        double[] extremism = new double[problem.getTotalPapers()];
//        
//        Objective objective = problem.getObjective();
//        
//        for (int p1 = 0; p1 < problem.getTotalPapers(); p1++) {
//            for (int p2 = p1 + 1; p2 < problem.getTotalPapers(); p2++) {
//                
//                double distance = objective.getDistance(p1, p2);
//                
//                //tranform it into an extremism metric
//                distance -= 0.5;
//                distance = distance * distance;
//                
//                extremism[p1] += distance;
//                extremism[p2] += distance;
//            }
//            
//            //We're done with p1 now
//            extremism[p1] /= problem.getTotalPapers();
//        }
//        
//        return extremism;
//    }

    private int getPaperMostDistantFor(int session, int track) {
        
        int startSlot = solution.getGlobalSlot(session, 0, 0);
        int stopSlot = solution.getGlobalSlot(session, track - 1, problem.getPapersInSession() - 1);
        
        int bestPaper = -1;
        double bestScore = -1;
        
        for (int p1 = 0; p1 < problem.getTotalPapers(); p1++) {
            int currentSlot = solution.getGlobalSlot(p1);
            if (currentSlot != -1) {
                continue; //already assigned
            }
            
            double score = 0;
            //Test against every paper already assigned to this session
            for (int slot = startSlot; slot < stopSlot; slot++) {
                int placedPaper = solution.getPaper(slot);
                score += objective.getDistance(p1, placedPaper);
            }
            
            if (score > bestScore) {
                bestPaper = p1;
                bestScore = score;
            }
        }
        
        return bestPaper;
    }

    private int getPaperClosestTo(int session, int track) {
        int bestPaper = -1;
        double bestScore = -1;
        
        for (int p1 = 0; p1 < problem.getTotalPapers(); p1++) {
            int currentSlot = solution.getGlobalSlot(p1);
            if (currentSlot != -1) {
                continue; //already assigned
            }
            
            double score = 0;
            
            for (int k = 0; k < problem.getPapersInSession(); k++) {
                int paperInSession = solution.getPaper(session, track, k);
                if (paperInSession == -1) {
                    continue;
                }
                
                score += 1 - objective.getDistance(p1, paperInSession);
            }
            
            if (score > bestScore) {
                bestPaper = p1;
                bestScore = score;
            }
        }
        
        return bestPaper;
    }
}
