/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package conferencesolver.solution.improver;

import conferencesolver.problem.ConferenceProblem;
import conferencesolver.problem.Objective;
import conferencesolver.solution.ConferenceSolution;

/**
 *
 * @author Michael Brooks <mjbrooks@uw.edu>
 */
public abstract class AbstractSolutionImprover implements SolutionImprover {

    private int stepsTaken = 0;
    protected ConferenceSolution currentSolution;
    private double currentQuality;
    protected ConferenceProblem problem;
    protected ConferenceSolution startingSolution;
    protected Objective objective;
    
    @Override
    public int getStepsTaken() {
        return stepsTaken;
    }

    @Override
    public void initialize(ConferenceSolution startingSolution) {
        this.problem = startingSolution.getProblem();
        this.objective = problem.getObjective();
        this.startingSolution = startingSolution.deepCopy();
        this.currentSolution = startingSolution;
        
        this.internalInit();
    }

    protected abstract void internalInit();
    
    
    protected abstract boolean internalStep();
    
    @Override
    public boolean doStep(ConferenceSolution start) {
        this.currentSolution = start.deepCopy();
        boolean result = internalStep();
        this.stepsTaken++;
        return result;
    }

    @Override
    public void forceObjective(Objective objective) {
        this.objective = objective;
    }
    
    @Override
    public ConferenceSolution getCurrentSolution() {
        return this.currentSolution;
    }
    
    protected void setCurrentSolution(ConferenceSolution solution) {
        this.currentSolution = solution;
    }
    
    @Override
    public double getCurrentQuality() {
        return this.currentQuality;
    }
    
    protected void setCurrentQuality(double quality) {
        this.currentQuality = quality;
    }
}
