/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package conferencesolver.solution.improver;

import conferencesolver.problem.CachedObjective;
import conferencesolver.solution.ConferenceSolution;

/**
 *
 * @author Michael Brooks <mjbrooks@uw.edu>
 */
public class AlternatingSearch extends AbstractSolutionImprover {
    private final SolutionImprover local;
    private final SolutionImprover macro;
    private boolean defaultLocal;
    
    private boolean localStalled = false;
    private boolean macroStalled = false;
    private boolean currentIsLocal;
    
    public AlternatingSearch(SolutionImprover localTemplate, SolutionImprover macroTemplate) {
        this(localTemplate, macroTemplate, true);
    }
    
    public AlternatingSearch(SolutionImprover localTemplate, SolutionImprover macroTemplate, boolean defaultLocal) {
        this.local = localTemplate.copy();
        this.macro = macroTemplate.copy();
        this.defaultLocal = defaultLocal;
    }

    @Override
    public SolutionImprover copy() {
        return new AlternatingSearch(this.local, this.macro, defaultLocal);
    }

    @Override
    public void internalInit() {
        this.local.initialize(startingSolution);
        this.macro.initialize(startingSolution);
        localStalled = false;
        macroStalled = false;
        currentIsLocal = defaultLocal;
    }

    @Override
    public boolean internalStep() {

        if (!localStalled || !macroStalled) {
            if (currentIsLocal) {
                localStalled = !local.doStep(currentSolution);
                if (localStalled) {
                    currentIsLocal = false;
                } else {
                    //If local didn't stall, unstick macro
                    macroStalled = false;
                }
                
                this.setCurrentSolution(local.getCurrentSolution());
                this.setCurrentQuality(local.getCurrentQuality());
                
                return true;
            } else {
                macroStalled = !macro.doStep(currentSolution);
                if (macroStalled) {
                    currentIsLocal = true;
                } else {
                    //If macro didn't stall, unstick local
                    localStalled = false;
                }
                
                this.setCurrentSolution(macro.getCurrentSolution());
                this.setCurrentQuality(macro.getCurrentQuality());
                
                return true;
            }
        }
        
        return false;
    }
    
}
