/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package conferencesolver.solution.improver;

import conferencesolver.problem.CachedObjective;
import conferencesolver.solution.ConferenceSolution;

/**
 *
 * @author Michael Brooks <mjbrooks@uw.edu>
 */
public class BasicLocalSearch extends AbstractSolutionImprover {

    public BasicLocalSearch() {
    }

    @Override
    public SolutionImprover copy() {
        return new BasicLocalSearch();
    }

    @Override
    public void internalInit() {
        this.objective = new CachedObjective(problem, objective);
    }

    @Override
    public boolean internalStep() {

        //Look at every possible switch and see which is best
        Swap bestSwap = getSwap(currentSolution);

        double minAdmissableImprovement = ZERO_THRESHOLD;

        if (bestSwap != null && bestSwap.deltaScore > minAdmissableImprovement) {

            currentSolution.swapSlots(bestSwap.slot1, bestSwap.slot2);

            double score = this.getCurrentQuality() + bestSwap.deltaScore;

            int session1 = currentSolution.getSessionFor(bestSwap.slot1);
            int session2 = currentSolution.getSessionFor(bestSwap.slot2);

            CachedObjective obj = (CachedObjective) objective;
            obj.invalidate(session1);
            if (session2 != session1) {
                obj.invalidate(session2);
            }

            this.setCurrentSolution(currentSolution);
            this.setCurrentQuality(score);
            return true;
        } else {
            return false;
        }

    }
    private final double ZERO_THRESHOLD = 0.01;

    private Swap getSwap(ConferenceSolution solution) {
        double bestSwapScore = -Double.MAX_VALUE;
        int bestSlot1 = -1;
        int bestSlot2 = -1;

        for (int slot1 = 0; slot1 < problem.getTotalPapers(); slot1++) {
            for (int slot2 = slot1 + 1; slot2 < problem.getTotalPapers(); slot2++) {

                //Need to find the delta from switching the paper in slot1 with the paper in slot2
                double newScoreDelta = objective.getSwapValueDelta(solution, slot1, slot2);

                if (bestSwapScore < newScoreDelta) {
                    bestSwapScore = newScoreDelta;
                    bestSlot1 = slot1;
                    bestSlot2 = slot2;
                }
            }
        }

        if (bestSlot1 != -1) {
            return new Swap(bestSlot1, bestSlot2, bestSwapScore);
        } else {
            return null;
        }
    }
}
