/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package conferencesolver.solution.improver;

import conferencesolver.problem.CachedObjective;
import conferencesolver.solution.ConferenceSolution;

/**
 *
 * @author Michael Brooks <mjbrooks@uw.edu>
 */
public class BasicMacroSearch extends AbstractSolutionImprover {

    public BasicMacroSearch() {
    }

    @Override
    public SolutionImprover copy() {
        return new BasicMacroSearch();
    }

    @Override
    public void internalInit() {
        this.objective = new CachedObjective(problem, objective);
    }

    @Override
    public boolean internalStep() {

        //Look at every possible switch and see which is best
        TrackSwap bestSwap = getSwap(currentSolution);

        double minAdmissableImprovement = ZERO_THRESHOLD;

        if (bestSwap != null && bestSwap.deltaScore > minAdmissableImprovement) {

            currentSolution.swapTracks(bestSwap.session1, bestSwap.track1, bestSwap.session2, bestSwap.track2);

            double score = this.getCurrentQuality() + bestSwap.deltaScore;

            CachedObjective obj = (CachedObjective) objective;
            obj.invalidate(bestSwap.session1);
            if (bestSwap.session2 != bestSwap.session1) {
                obj.invalidate(bestSwap.session2);
            }

            this.setCurrentSolution(currentSolution);
            this.setCurrentQuality(score);
            return true;
        } else {
            return false;
        }

    }
    private final double ZERO_THRESHOLD = 0.01;

    private TrackSwap getSwap(ConferenceSolution solution) {
        double bestSwapScore = -Double.MAX_VALUE;
        int bestSession1 = -1;
        int bestTrack1 = -1;
        int bestSession2 = -1;
        int bestTrack2 = -1;

        for (int session1 = 0; session1 < problem.getNumSessions(); session1++) {
            for (int track1 = 0; track1 < problem.getTracksInSession(); track1++) {
                
                for (int session2 = session1 + 1; session2 < problem.getNumSessions(); session2++) {
                    for (int track2 = 0; track2 < problem.getTracksInSession(); track2++) {
                        
                        double newScoreDelta = objective.getTrackSwapValueDelta(solution, session1, track1, session2, track2);
                        
                        if (bestSwapScore < newScoreDelta) {
                            bestSwapScore = newScoreDelta;
                            
                            bestSession1 = session1;
                            bestTrack1 = track1;
                            bestSession2 = session2;
                            bestTrack2 = track2;
                        }
                        
                    }
                }
                
            }
        }
        
        if (bestSession1 != -1) {
            return new TrackSwap(bestSession1, bestTrack1, bestSession2, bestTrack2, bestSwapScore);
        } else {
            return null;
        }
    }
}
