/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package conferencesolver.solution.improver;

import conferencesolver.problem.CachedObjective;
import conferencesolver.solution.ConferenceSolution;
import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedList;

/**
 *
 * @author Michael Brooks <mjbrooks@uw.edu>
 */
public class TabuLocalSearch extends AbstractSolutionImprover {

    private final int tabuListLength;
    private LinkedList<String> tabuList;

    public TabuLocalSearch(int tabuListLength) {
        this.tabuListLength = tabuListLength;
    }
    private final double ZERO_THRESHOLD = 0.01;

    private ArrayList<Swap> getSwaps(ConferenceSolution solution, double minAdmissableImprovement) {

        ArrayList<Swap> swaps = new ArrayList<Swap>();

        for (int slot1 = 0; slot1 < problem.getTotalPapers(); slot1++) {
            for (int slot2 = slot1 + 1; slot2 < problem.getTotalPapers(); slot2++) {

                //Need to find the delta from switching the paper in slot1 with the paper in slot2
                double newScoreDelta = objective.getSwapValueDelta(solution, slot1, slot2);

                if (newScoreDelta > minAdmissableImprovement) {
                    swaps.add(new Swap(slot1, slot2, newScoreDelta));
                }
            }
        }

        return swaps;
    }

    @Override
    public SolutionImprover copy() {
        return new TabuLocalSearch(tabuListLength);
    }

    @Override
    public void internalInit() {
        this.objective = new CachedObjective(problem, objective);
        tabuList = new LinkedList<String>();
    }

    @Override
    public boolean internalStep() {

        //Look at every possible switch and see which is best
        ArrayList<Swap> bestSwaps = getSwaps(currentSolution, ZERO_THRESHOLD);

        Collections.sort(bestSwaps);

        Swap bestSwap = chooseBestSwap(bestSwaps);

        if (bestSwap != null) {

            currentSolution.swapSlots(bestSwap.slot1, bestSwap.slot2);

            double score = this.getCurrentQuality() + bestSwap.deltaScore;

            int session1 = currentSolution.getSessionFor(bestSwap.slot1);
            int session2 = currentSolution.getSessionFor(bestSwap.slot2);

            CachedObjective obj = (CachedObjective) objective;
            obj.invalidate(session1);
            if (session2 != session1) {
                obj.invalidate(session2);
            }

            tabuList.addLast(currentSolution.toString());
            if (tabuList.size() > tabuListLength) {
                tabuList.removeFirst();
            }

            this.setCurrentSolution(currentSolution);
            this.setCurrentQuality(score);
            return true;
        } else {
            return false;
        }
    }

    private Swap chooseBestSwap(ArrayList<Swap> swaps) {
        
        //Go down the list to choose the best one
        for (int i = swaps.size() - 1; i >= 0; i--) {
            
            Swap possibleSwap = swaps.get(i);
            
            //Get the solution id
            currentSolution.swapSlots(possibleSwap.slot1, possibleSwap.slot2);
            String solutionId = currentSolution.toString();
            currentSolution.swapSlots(possibleSwap.slot1, possibleSwap.slot2); //Unswap
            if (this.tabuList.contains(solutionId)) {
                //Skip tabu solutions
                continue;
            }
            
            return possibleSwap;
        }
        
        return null;
    }
}
